==> apache-druid-34.0.0-src.tar.gz: apache-druid-34.0.0-src/integration-tests/src/main/java/org/apache/druid/server/coordination/ServerManagerForQueryErrorTest.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.druid.server.coordination;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.inject.Inject;
import org.apache.commons.lang3.mutable.MutableBoolean;
import org.apache.druid.client.cache.Cache;
import org.apache.druid.client.cache.CacheConfig;
import org.apache.druid.client.cache.CachePopulator;
import org.apache.druid.guice.annotations.Smile;
import org.apache.druid.java.util.common.guava.Accumulator;
import org.apache.druid.java.util.common.guava.FunctionalIterable;
import org.apache.druid.java.util.common.guava.Sequence;
import org.apache.druid.java.util.common.guava.Yielder;
import org.apache.druid.java.util.common.guava.YieldingAccumulator;
import org.apache.druid.java.util.common.io.Closer;
import org.apache.druid.java.util.common.logger.Logger;
import org.apache.druid.java.util.emitter.service.ServiceEmitter;
import org.apache.druid.query.Query;
import org.apache.druid.query.QueryCapacityExceededException;
import org.apache.druid.query.QueryContext;
import org.apache.druid.query.QueryProcessingPool;
import org.apache.druid.query.QueryRunner;
import org.apache.druid.query.QueryRunnerFactory;
import org.apache.druid.query.QueryRunnerFactoryConglomerate;
import org.apache.druid.query.QueryTimeoutException;
import org.apache.druid.query.QueryToolChest;
import org.apache.druid.query.QueryUnsupportedException;
import org.apache.druid.query.ReportTimelineMissingSegmentQueryRunner;
import org.apache.druid.query.ResourceLimitExceededException;
import org.apache.druid.query.SegmentDescriptor;
import org.apache.druid.query.policy.NoopPolicyEnforcer;
import org.apache.druid.segment.ReferenceCountedSegmentProvider;
import org.apache.druid.segment.SegmentMapFunction;
import org.apache.druid.server.SegmentManager;
import org.apache.druid.server.initialization.ServerConfig;
import org.apache.druid.timeline.VersionedIntervalTimeline;

import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicLong;

/**
 * This server manager is designed to test various query failures.
 * <ul>
 *  <li> Missing segments. A segment can be missing during a query if a historical drops the segment
 *   after the broker issues the query to the historical. To mimic this situation, the historical
 *   with this server manager announces all segments assigned, and reports missing segments based on the following:
 *   <ul>
 *     <li> If {@link #QUERY_RETRY_UNAVAILABLE_SEGMENT_IDX_KEY} and {@link #QUERY_RETRY_TEST_CONTEXT_KEY} are set,
 *           the segment at that index is reported as missing exactly once.</li>
 *     <li> If {@link #QUERY_RETRY_UNAVAILABLE_SEGMENT_IDX_KEY} is not set or is -1, it simulates missing segments
 *     starting from the beginning, up to {@link #MAX_NUM_FALSE_MISSING_SEGMENTS_REPORTS}.</li>
 *   </ul>
 *   The missing report is only generated once for the first time. Post that report, upon retry, all segments are served
 *   for the datasource. See ITQueryRetryTestOnMissingSegments. </li>
 * <li> Other query errors. This server manager returns a sequence that always throws an exception
 *   based on a given query context value. See ITQueryErrorTest. </li>
 * </ul>
 *
 * @see org.apache.druid.query.RetryQueryRunner for query retrying.
 * @see org.apache.druid.client.JsonParserIterator for handling query errors from historicals.
 */
public class ServerManagerForQueryErrorTest extends ServerManager
{
  // Query context key that indicates this query is for query retry testing.
  public static final String QUERY_RETRY_TEST_CONTEXT_KEY = "query-retry-test";
  public static final String QUERY_TIMEOUT_TEST_CONTEXT_KEY = "query-timeout-test";
  public static final String QUERY_CAPACITY_EXCEEDED_TEST_CONTEXT_KEY = "query-capacity-exceeded-test";
  public static final String QUERY_UNSUPPORTED_TEST_CONTEXT_KEY = "query-unsupported-test";
  public static final String RESOURCE_LIMIT_EXCEEDED_TEST_CONTEXT_KEY = "resource-limit-exceeded-test";
  public static final String QUERY_FAILURE_TEST_CONTEXT_KEY = "query-failure-test";
  /**
   * Query context that indicates which segment should be marked as unavilable/missing.
   * This should be used in conjunction with {@link #QUERY_RETRY_TEST_CONTEXT_KEY}.
   * <p>
   * A value of {@code 0} means the first segment will be reported as missing, {@code 1} for the second, and so on.
   * If this key is not set (default = -1), the test will instead simulate missing up to
   * {@link #MAX_NUM_FALSE_MISSING_SEGMENTS_REPORTS} segments from the beginning.
   * </p>
   */
  public static final String QUERY_RETRY_UNAVAILABLE_SEGMENT_IDX_KEY = "unavailable-segment-idx";
  private static final int MAX_NUM_FALSE_MISSING_SEGMENTS_REPORTS = 1;

  private static final Logger LOG = new Logger(ServerManagerForQueryErrorTest.class);

  private final ConcurrentHashMap<String, Integer> queryToIgnoredSegments = new ConcurrentHashMap<>();

  @Inject
  public ServerManagerForQueryErrorTest(
      QueryRunnerFactoryConglomerate conglomerate,
      ServiceEmitter emitter,
      QueryProcessingPool queryProcessingPool,
      CachePopulator cachePopulator,
      @Smile ObjectMapper objectMapper,
      Cache cache,
      CacheConfig cacheConfig,
      SegmentManager segmentManager,
      ServerConfig serverConfig
  )
  {
    super(
        conglomerate,
        emitter,
        queryProcessingPool,
        cachePopulator,
        objectMapper,
        cache,
        cacheConfig,
        segmentManager,
        serverConfig,
        NoopPolicyEnforcer.instance()
    );
  }

  @Override
  protected <T> FunctionalIterable<QueryRunner<T>> getQueryRunnersForSegments(
      final VersionedIntervalTimeline<String, ReferenceCountedSegmentProvider> timeline,
      final Iterable<SegmentDescriptor> specs,
      final Query<T> query,
      final QueryRunnerFactory<T, Query<T>> factory,
      final QueryToolChest<T, Query<T>> toolChest,
      final SegmentMapFunction segmentMapFn,
      final AtomicLong cpuTimeAccumulator,
      final Optional<byte[]> cacheKeyPrefix,
      final Closer closer
  )
  {
    return FunctionalIterable
        .create(acquireAllSegments(timeline, specs, segmentMapFn, closer))
        .transform(
            ref ->
                ref.getSegmentReference()
                   .map(segment -> {
                          final QueryContext queryContext = query.context();
                          if (queryContext.getBoolean(QUERY_RETRY_TEST_CONTEXT_KEY, false)) {
                            final int unavailableSegmentIdx = queryContext.getInt(QUERY_RETRY_UNAVAILABLE_SEGMENT_IDX_KEY, -1);
                            final MutableBoolean isIgnoreSegment = new MutableBoolean(false);
                            queryToIgnoredSegments.compute(
                                query.getMostSpecificId(),
                                (queryId, ignoreCounter) -> {
                                  if (ignoreCounter == null) {
                                    ignoreCounter = 0;
                                  }

                                  if (unavailableSegmentIdx >= 0 && unavailableSegmentIdx == ignoreCounter) {
                                    // Fail exactly once when counter matches the configured retry index
                                    ignoreCounter++;
                                    isIgnoreSegment.setTrue();
                                  } else if (ignoreCounter < MAX_NUM_FALSE_MISSING_SEGMENTS_REPORTS) {
                                    // Fail up to N times for this query
                                    ignoreCounter++;
                                    isIgnoreSegment.setTrue();
                                  }
                                  return ignoreCounter;
                                }
                            );

                            if (isIgnoreSegment.isTrue()) {
                              LOG.info(
                                  "Pretending I don't have segment[%s]",
                                  ref.getSegmentDescriptor()
                              );
                              return new ReportTimelineMissingSegmentQueryRunner<T>(ref.getSegmentDescriptor());
                            }
                          } else if (queryContext.getBoolean(QUERY_TIMEOUT_TEST_CONTEXT_KEY, false)) {
                            return (QueryRunner<T>) (queryPlus, responseContext) -> new Sequence<>()
                            {
                              @Override
                              public <OutType> OutType accumulate(
                                  OutType initValue,
                                  Accumulator<OutType, T> accumulator
                              )
                              {
                                throw new QueryTimeoutException("query timeout test");
                              }

                              @Override
                              public <OutType> Yielder<OutType> toYielder(
                                  OutType initValue,
                                  YieldingAccumulator<OutType, T> accumulator
                              )
                              {
                                throw new QueryTimeoutException("query timeout test");
                              }
                            };
                          } else if (queryContext.getBoolean(QUERY_CAPACITY_EXCEEDED_TEST_CONTEXT_KEY, false)) {
                            return (QueryRunner<T>) (queryPlus, responseContext) -> new Sequence<>()
                            {
                              @Override
                              public <OutType> OutType accumulate(
                                  OutType initValue,
                                  Accumulator<OutType, T> accumulator
                              )
                              {
                                throw QueryCapacityExceededException.withErrorMessageAndResolvedHost(
                                    "query capacity exceeded test"
                                );
                              }

                              @Override
                              public <OutType> Yielder<OutType> toYielder(
                                  OutType initValue,
                                  YieldingAccumulator<OutType, T> accumulator
                              )
                              {
                                throw QueryCapacityExceededException.withErrorMessageAndResolvedHost(
                                    "query capacity exceeded test"
                                );
                              }
                            };
                          } else if (queryContext.getBoolean(QUERY_UNSUPPORTED_TEST_CONTEXT_KEY, false)) {
                            return (QueryRunner<T>) (queryPlus, responseContext) -> new Sequence<>()
                            {
                              @Override
                              public <OutType> OutType accumulate(
                                  OutType initValue,
                                  Accumulator<OutType, T> accumulator
                              )
                              {
                                throw new QueryUnsupportedException("query unsupported test");
                              }

                              @Override
                              public <OutType> Yielder<OutType> toYielder(
                                  OutType initValue,
                                  YieldingAccumulator<OutType, T> accumulator
                              )
                              {
                                throw new QueryUnsupportedException("query unsupported test");
                              }
                            };
                          } else if (queryContext.getBoolean(RESOURCE_LIMIT_EXCEEDED_TEST_CONTEXT_KEY, false)) {
                            return (QueryRunner<T>) (queryPlus, responseContext) -> new Sequence<>()
                            {
                              @Override
                              public <OutType> OutType accumulate(
                                  OutType initValue,
                                  Accumulator<OutType, T> accumulator
                              )
                              {
                                throw new ResourceLimitExceededException("resource limit exceeded test");
                              }

                              @Override
                              public <OutType> Yielder<OutType> toYielder(
                                  OutType initValue,
                                  YieldingAccumulator<OutType, T> accumulator
                              )
                              {
                                throw new ResourceLimitExceededException("resource limit exceeded test");
                              }
                            };
                          } else if (queryContext.getBoolean(QUERY_FAILURE_TEST_CONTEXT_KEY, false)) {
                            return (QueryRunner<T>) (queryPlus, responseContext) -> new Sequence<>()
                            {
                              @Override
                              public <OutType> OutType accumulate(
                                  OutType initValue,
                                  Accumulator<OutType, T> accumulator
                              )
                              {
                                throw new RuntimeException("query failure test");
                              }

                              @Override
                              public <OutType> Yielder<OutType> toYielder(
                                  OutType initValue,
                                  YieldingAccumulator<OutType, T> accumulator
                              )
                              {
                                throw new RuntimeException("query failure test");
                              }
                            };
                          }

                          return buildQueryRunnerForSegment(
                              ref.getSegmentDescriptor(),
                              segment,
                              factory,
                              toolChest,
                              cpuTimeAccumulator,
                              cacheKeyPrefix
                          );
                        }
                   ).orElse(
                       new ReportTimelineMissingSegmentQueryRunner<>(ref.getSegmentDescriptor())
                   )
        );
  }
}

==> apache-druid-35.0.1-src.tar.gz: apache-druid-35.0.1-src/integration-tests/src/main/java/org/apache/druid/server/coordination/ServerManagerForQueryErrorTest.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.druid.server.coordination;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.inject.Inject;
import org.apache.commons.lang3.mutable.MutableBoolean;
import org.apache.druid.client.cache.Cache;
import org.apache.druid.client.cache.CacheConfig;
import org.apache.druid.client.cache.CachePopulator;
import org.apache.druid.guice.annotations.Smile;
import org.apache.druid.java.util.common.guava.Accumulator;
import org.apache.druid.java.util.common.guava.FunctionalIterable;
import org.apache.druid.java.util.common.guava.Sequence;
import org.apache.druid.java.util.common.guava.Yielder;
import org.apache.druid.java.util.common.guava.YieldingAccumulator;
import org.apache.druid.java.util.common.io.Closer;
import org.apache.druid.java.util.common.logger.Logger;
import org.apache.druid.java.util.emitter.service.ServiceEmitter;
import org.apache.druid.query.Query;
import org.apache.druid.query.QueryCapacityExceededException;
import org.apache.druid.query.QueryContext;
import org.apache.druid.query.QueryProcessingPool;
import org.apache.druid.query.QueryRunner;
import org.apache.druid.query.QueryRunnerFactory;
import org.apache.druid.query.QueryRunnerFactoryConglomerate;
import org.apache.druid.query.QueryTimeoutException;
import org.apache.druid.query.QueryToolChest;
import org.apache.druid.query.QueryUnsupportedException;
import org.apache.druid.query.ReportTimelineMissingSegmentQueryRunner;
import org.apache.druid.query.ResourceLimitExceededException;
import org.apache.druid.query.SegmentDescriptor;
import org.apache.druid.query.policy.NoopPolicyEnforcer;
import org.apache.druid.segment.SegmentMapFunction;
import org.apache.druid.server.SegmentManager;
import org.apache.druid.server.ServerManager;
import org.apache.druid.server.initialization.ServerConfig;
import org.apache.druid.timeline.DataSegment;
import org.apache.druid.timeline.VersionedIntervalTimeline;

import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicLong;

/**
 * This server manager is designed to test various query failures.
 * <ul>
 *  <li> Missing segments. A segment can be missing during a query if a historical drops the segment
 *   after the broker issues the query to the historical. To mimic this situation, the historical
 *   with this server manager announces all segments assigned, and reports missing segments based on the following:
 *   <ul>
 *     <li> If {@link #QUERY_RETRY_UNAVAILABLE_SEGMENT_IDX_KEY} and {@link #QUERY_RETRY_TEST_CONTEXT_KEY} are set,
 *           the segment at that index is reported as missing exactly once.</li>
 *     <li> If {@link #QUERY_RETRY_UNAVAILABLE_SEGMENT_IDX_KEY} is not set or is -1, it simulates missing segments
 *     starting from the beginning, up to {@link #MAX_NUM_FALSE_MISSING_SEGMENTS_REPORTS}.</li>
 *   </ul>
 *   The missing report is only generated once for the first time. Post that report, upon retry, all segments are served
 *   for the datasource. See ITQueryRetryTestOnMissingSegments. </li>
 * <li> Other query errors. This server manager returns a sequence that always throws an exception
 *   based on a given query context value. See ITQueryErrorTest. </li>
 * </ul>
 *
 * @see org.apache.druid.query.RetryQueryRunner for query retrying.
 * @see org.apache.druid.client.JsonParserIterator for handling query errors from historicals.
 */
public class ServerManagerForQueryErrorTest extends ServerManager
{
  // Query context key that indicates this query is for query retry testing.
  public static final String QUERY_RETRY_TEST_CONTEXT_KEY = "query-retry-test";
  public static final String QUERY_TIMEOUT_TEST_CONTEXT_KEY = "query-timeout-test";
  public static final String QUERY_CAPACITY_EXCEEDED_TEST_CONTEXT_KEY = "query-capacity-exceeded-test";
  public static final String QUERY_UNSUPPORTED_TEST_CONTEXT_KEY = "query-unsupported-test";
  public static final String RESOURCE_LIMIT_EXCEEDED_TEST_CONTEXT_KEY = "resource-limit-exceeded-test";
  public static final String QUERY_FAILURE_TEST_CONTEXT_KEY = "query-failure-test";
  /**
   * Query context that indicates which segment should be marked as unavilable/missing.
   * This should be used in conjunction with {@link #QUERY_RETRY_TEST_CONTEXT_KEY}.
   * <p>
   * A value of {@code 0} means the first segment will be reported as missing, {@code 1} for the second, and so on.
   * If this key is not set (default = -1), the test will instead simulate missing up to
   * {@link #MAX_NUM_FALSE_MISSING_SEGMENTS_REPORTS} segments from the beginning.
   * </p>
   */
  public static final String QUERY_RETRY_UNAVAILABLE_SEGMENT_IDX_KEY = "unavailable-segment-idx";
  private static final int MAX_NUM_FALSE_MISSING_SEGMENTS_REPORTS = 1;

  private static final Logger LOG = new Logger(ServerManagerForQueryErrorTest.class);

  private final ConcurrentHashMap<String, Integer> queryToIgnoredSegments = new ConcurrentHashMap<>();

  @Inject
  public ServerManagerForQueryErrorTest(
      QueryRunnerFactoryConglomerate conglomerate,
      ServiceEmitter emitter,
      QueryProcessingPool queryProcessingPool,
      CachePopulator cachePopulator,
      @Smile ObjectMapper objectMapper,
      Cache cache,
      CacheConfig cacheConfig,
      SegmentManager segmentManager,
      ServerConfig serverConfig
  )
  {
    super(
        conglomerate,
        emitter,
        queryProcessingPool,
        cachePopulator,
        objectMapper,
        cache,
        cacheConfig,
        segmentManager,
        serverConfig,
        NoopPolicyEnforcer.instance()
    );
  }

  @Override
  protected <T> FunctionalIterable<QueryRunner<T>> getQueryRunnersForSegments(
      final VersionedIntervalTimeline<String, DataSegment> timeline,
      final Iterable<SegmentDescriptor> specs,
      final Query<T> query,
      final QueryRunnerFactory<T, Query<T>> factory,
      final QueryToolChest<T, Query<T>> toolChest,
      final SegmentMapFunction segmentMapFn,
      final AtomicLong cpuTimeAccumulator,
      final Optional<byte[]> cacheKeyPrefix,
      final Closer closer
  )
  {
    return FunctionalIterable
        .create(getSegmentReferences(timeline, specs, segmentMapFn, query.context().getTimeout()))
        .transform(
            ref ->
                ref.getSegmentReference()
                   .map(segment -> {
                          final QueryContext queryContext = query.context();
                          if (queryContext.getBoolean(QUERY_RETRY_TEST_CONTEXT_KEY, false)) {
                            final int unavailableSegmentIdx = queryContext.getInt(QUERY_RETRY_UNAVAILABLE_SEGMENT_IDX_KEY, -1);
                            final MutableBoolean isIgnoreSegment = new MutableBoolean(false);
                            queryToIgnoredSegments.compute(
                                query.getMostSpecificId(),
                                (queryId, ignoreCounter) -> {
                                  if (ignoreCounter == null) {
                                    ignoreCounter = 0;
                                  }

                                  if (unavailableSegmentIdx >= 0 && unavailableSegmentIdx == ignoreCounter) {
                                    // Fail exactly once when counter matches the configured retry index
                                    ignoreCounter++;
                                    isIgnoreSegment.setTrue();
                                  } else if (ignoreCounter < MAX_NUM_FALSE_MISSING_SEGMENTS_REPORTS) {
                                    // Fail up to N times for this query
                                    ignoreCounter++;
                                    isIgnoreSegment.setTrue();
                                  }
                                  return ignoreCounter;
                                }
                            );

                            if (isIgnoreSegment.isTrue()) {
                              LOG.info(
                                  "Pretending I don't have segment[%s]",
                                  ref.getSegmentDescriptor()
                              );
                              return new ReportTimelineMissingSegmentQueryRunner<T>(ref.getSegmentDescriptor());
                            }
                          } else if (queryContext.getBoolean(QUERY_TIMEOUT_TEST_CONTEXT_KEY, false)) {
                            return (QueryRunner<T>) (queryPlus, responseContext) -> new Sequence<>()
                            {
                              @Override
                              public <OutType> OutType accumulate(
                                  OutType initValue,
                                  Accumulator<OutType, T> accumulator
                              )
                              {
                                throw new QueryTimeoutException("query timeout test");
                              }

                              @Override
                              public <OutType> Yielder<OutType> toYielder(
                                  OutType initValue,
                                  YieldingAccumulator<OutType, T> accumulator
                              )
                              {
                                throw new QueryTimeoutException("query timeout test");
                              }
                            };
                          } else if (queryContext.getBoolean(QUERY_CAPACITY_EXCEEDED_TEST_CONTEXT_KEY, false)) {
                            return (QueryRunner<T>) (queryPlus, responseContext) -> new Sequence<>()
                            {
                              @Override
                              public <OutType> OutType accumulate(
                                  OutType initValue,
                                  Accumulator<OutType, T> accumulator
                              )
                              {
                                throw QueryCapacityExceededException.withErrorMessageAndResolvedHost(
                                    "query capacity exceeded test"
                                );
                              }

                              @Override
                              public <OutType> Yielder<OutType> toYielder(
                                  OutType initValue,
                                  YieldingAccumulator<OutType, T> accumulator
                              )
                              {
                                throw QueryCapacityExceededException.withErrorMessageAndResolvedHost(
                                    "query capacity exceeded test"
                                );
                              }
                            };
                          } else if (queryContext.getBoolean(QUERY_UNSUPPORTED_TEST_CONTEXT_KEY, false)) {
                            return (QueryRunner<T>) (queryPlus, responseContext) -> new Sequence<>()
                            {
                              @Override
                              public <OutType> OutType accumulate(
                                  OutType initValue,
                                  Accumulator<OutType, T> accumulator
                              )
                              {
                                throw new QueryUnsupportedException("query unsupported test");
                              }

                              @Override
                              public <OutType> Yielder<OutType> toYielder(
                                  OutType initValue,
                                  YieldingAccumulator<OutType, T> accumulator
                              )
                              {
                                throw new QueryUnsupportedException("query unsupported test");
                              }
                            };
                          } else if (queryContext.getBoolean(RESOURCE_LIMIT_EXCEEDED_TEST_CONTEXT_KEY, false)) {
                            return (QueryRunner<T>) (queryPlus, responseContext) -> new Sequence<>()
                            {
                              @Override
                              public <OutType> OutType accumulate(
                                  OutType initValue,
                                  Accumulator<OutType, T> accumulator
                              )
                              {
                                throw new ResourceLimitExceededException("resource limit exceeded test");
                              }

                              @Override
                              public <OutType> Yielder<OutType> toYielder(
                                  OutType initValue,
                                  YieldingAccumulator<OutType, T> accumulator
                              )
                              {
                                throw new ResourceLimitExceededException("resource limit exceeded test");
                              }
                            };
                          } else if (queryContext.getBoolean(QUERY_FAILURE_TEST_CONTEXT_KEY, false)) {
                            return (QueryRunner<T>) (queryPlus, responseContext) -> new Sequence<>()
                            {
                              @Override
                              public <OutType> OutType accumulate(
                                  OutType initValue,
                                  Accumulator<OutType, T> accumulator
                              )
                              {
                                throw new RuntimeException("query failure test");
                              }

                              @Override
                              public <OutType> Yielder<OutType> toYielder(
                                  OutType initValue,
                                  YieldingAccumulator<OutType, T> accumulator
                              )
                              {
                                throw new RuntimeException("query failure test");
                              }
                            };
                          }

                          return buildQueryRunnerForSegment(
                              ref.getSegmentDescriptor(),
                              segment,
                              factory,
                              toolChest,
                              cpuTimeAccumulator,
                              cacheKeyPrefix
                          );
                        }
                   ).orElse(
                       new ReportTimelineMissingSegmentQueryRunner<>(ref.getSegmentDescriptor())
                   )
        );
  }
}
